<!--
 * @Date: 2024-05-04 16:37:48
 * @LastEditors: gaoshp
 * @LastEditTime: 2024-11-07 21:45:28
 * @FilePath: /cps-web/src/views/console/base/Add.vue
-->
<template>
    <el-container>
        <el-main>
            <el-row>
                <el-form style="width: 100%;" :model="form" :rules="rules" ref="dialogForm" label-width="120px"
                    label-position="center">
                    <el-row>
                        <el-col :span="8">
                            <el-form-item label="生产日历编号" prop="code">
                                <el-input :disabled="disabled" v-model="form.code" placeholder="请输入生产日历编号"
                                    clearable></el-input>
                            </el-form-item>
                        </el-col>
                        <el-col :span="8">
                            <el-form-item label="生产日历名称" prop="name">
                                <el-input :disabled="disabled" v-model="form.name" placeholder="请输入生产日历名称"
                                    clearable></el-input>
                            </el-form-item>
                        </el-col>
                        <el-col :span="8">
                            <el-form-item label="应用年份" prop="year">
                                <el-select :disabled="disabled" v-model="form.year" placeholder="" size=""
                                    @change="changeYear">
                                    <el-option v-for="item in options" :key="item.value" :label="item.label"
                                        :value="item.value" />
                                </el-select>
                            </el-form-item>
                        </el-col>
                        <el-col :span="8">
                            <el-form-item label="应用日期" prop="date">
                                <el-date-picker v-model="form.date" type="daterange" value-format="YYYY-MM-DD"
                                    start-placeholder="开始日期" end-placeholder="结束日期" @change="changeDate" />
                            </el-form-item>
                        </el-col>
                        <el-col :span="8" :offset="1">
                            <!-- <el-button-group>
                                <el-button :type="activeBtn == 0 ? 'primary' : ''"
                                    @click="activeBtn = 0">按规律选中日期</el-button>
                                <el-button :type="activeBtn == 1 ? 'primary' : ''" @click="activeBtn = 1">
                                    按间隔选中日期
                                </el-button>
                            </el-button-group> -->
                        </el-col>
                        <el-col :offset="1">
                            <el-radio-group v-model="holiday">
                                <el-radio :label="0">包含假期</el-radio>
                                <el-radio :label="1">不含假期</el-radio>
                                <el-radio :label="2">仅含假期</el-radio>
                            </el-radio-group>
                        </el-col>
                        <el-col :span="4" :offset="1">
                            <el-checkbox-group v-model="checkboxGroup" size="small" style="width: 200px"
                                @change="change">
                                <el-checkbox-button v-for="item in quicks" :key="item.value" :label="item.value">
                                    {{ item.label }}
                                </el-checkbox-button>
                            </el-checkbox-group>

                        </el-col>
                        <el-col :span="8">
                            <el-checkbox-group v-model="checkboxGroup1" size="small" style="width: 400px">
                                <el-checkbox-button v-for="item in weeks" :key="item.value" :label="item.value">
                                    {{ item.label }}
                                </el-checkbox-button>
                            </el-checkbox-group>
                        </el-col>
                    </el-row>
                    <el-row v-if="months.length > 0">
                        <el-col :span="6" v-for="(item, i) in months" :key="i" style="padding:  10px;">
                            <!-- <Calendar :min-date="item.start" :rows="1" :attributes="item.attributes"
                            @dayclick="dayclick($event, item)" /> -->
                            <Calendar :initial-page="{ month: item.month + 1, year: item.year }" :rows="1"
                                disable-page-swipe :nav-visibility="''" :attributes='item.attributes'
                                @dayclick="dayclick($event, item)" />
                        </el-col>
                    </el-row>
                </el-form>
            </el-row>
        </el-main>
        <el-footer>
            <el-row style="align-items: center;">
                <el-col :span="2">
                    班制方案
                </el-col>
                <el-col :span="4">
                    <el-select v-model="modelId" placeholder="" size="">
                        <el-option v-for="item in shiftList" :key="item.id" :label="item.name" :value="item.id">{{
                            item.name }}</el-option>
                    </el-select>
                </el-col>
                <el-col :span="2">
                    当前排班优先
                </el-col>
                <el-col :span="4">
                    <el-switch v-model="isHighPriority" @change="changeHigh" active-text="优先" inactive-text="不优先" />
                </el-col>
                <el-col :span="2">
                    <el-button type="primary" @click="plan">排班</el-button>
                </el-col>
                <el-col :span="2">
                    <el-button type="" @click="planCancel">取消排班</el-button>
                </el-col>
                <el-col :span="2">
                    <el-button type="primary" @click="save">保存</el-button>
                </el-col>
            </el-row>

        </el-footer>

    </el-container>
</template>

<script>
import moment from 'moment'
import { Calendar, DatePicker } from 'v-calendar';
import color from '@/utils/color'
export default {
    components: {
        Calendar,
        DatePicker
    },
    props: {
        shiftList: {
            type: Array,
            default: () => {
                return []
            }
        },
        drawerInfo: {
            type: Object,
            default: () => {
                return {}
            }
        },
    },
    data() {
        return {
            currentDate: '',
            currentDateItem: null,
            disabled: false,
            modelId: '',
            isHighPriority: false,
            dateDTOList: [],
            selectAttribute: [{ dot: true }],
            activeBtn: 0,
            checkboxGroup: [],
            checkboxGroup1: [],
            holiday: 0,
            months: [
            ],
            quicks: [{
                label: '工作日',
                value: 0
            }, {
                label: '周末',
                value: 1
            }],
            weeks: [
                {
                    label: '一',
                    value: 1
                },
                {
                    label: '二',
                    value: 2
                },
                {
                    label: '三',
                    value: 3
                },
                {
                    label: '四',
                    value: 4
                },
                {
                    label: '五',
                    value: 5
                },
                {
                    label: '六',
                    value: 6
                },
                {
                    label: '日',
                    value: 0
                },
            ],
            options: [],
            form: {
                year: ''
            },
            rules: {
                name: [
                    { required: true, message: '' }
                ],
                code: [
                    { required: true, message: '' }
                ],
                year: [
                    { required: true, message: '' }
                ],
            }
        }
    },
    watch: {

        holiday() {
            this.setDay()
        },
        checkboxGroup1() {
            this.setDay()
        },
        'form.year'(val) {
            this.months = []
            let currentYear = moment(val, 'YYYY').format('YYYY') - 0

            this.form.date = [`${currentYear}-01-01`, `${currentYear}-12-31`]
            if (currentYear == moment().format('YYYY')) {
                this.form.date[0] = moment().format('YYYY-MM-DD')
            }
            // console.log(this.months)
            this.$HTTP.post('/api/smis/shift-off-day/list', {
                year: currentYear,
                startOffDay: `${currentYear}-01-01`,
                endOffDay: `${currentYear}-12-31`
            }).then(res => {
                if (res.code === 200) {
                    this.offDays = res.data
                }
                let arr = []
                this.offDays.forEach(v => {
                    arr.push(v.startOffDay)
                    let day = moment(v.startOffDay)
                    while (day.add(1, 'd').unix() <= moment(v.endOffDay).unix()) {
                        arr.push(day.format('YYYY-MM-DD'))
                    }
                })
                for (let i = 0; i < 12; i++) {
                    this.months.push({
                        year: currentYear,
                        month: i,
                        id: `${currentYear}${i}`,
                        min: new Date(`${currentYear}-01-01`),
                        attributes: [
                            {
                                dot: 'yellow',
                                dates: arr.filter(item => {
                                    // console.log(moment(item).month(), 'mmonth', item)
                                    return moment(item).month() === i
                                })
                            },
                            {
                                highlight: true,
                                dates: [],
                            }
                        ]
                    })

                }
            })
        },
        shiftList() {
            this.modelId = this.shiftList[0].id
        }
    },
    created() {
        this.init()
    },
    methods: {
        init() {
            if (!this.drawerInfo.id) {
                let len = 15
                let currentYear = moment().format('YYYY') - 0
                this.form.year = currentYear
                for (let i = 0; i < len; i++) {
                    this.options.push({
                        label: currentYear + i,
                        value: currentYear + i
                    })
                }
                this.form.date = [moment().format('YYYY-MM-DD'), moment(`${currentYear}-12-31`).format('YYYY-MM-DD')]
            } else {
                this.disabled = true
                this.form.code = this.drawerInfo.code
                this.form.name = this.drawerInfo.name
                this.form.year = moment(this.drawerInfo.calendarDayVOList[0].calendarDate, 'YYYY-MM-DD').format('YYYY') - 0
                setTimeout(() => {
                    let currentYear = moment().format('YYYY') - 0
                    this.form.date = [`${currentYear}-01-01`, `${currentYear}-12-31`]
                    if (currentYear == this.form.year) {
                        this.form.date[0] = moment().format('YYYY-MM-DD')
                    }
                    [...new Set(this.drawerInfo.calendarDayVOList.map(v => v.modelId))].forEach(item => {
                        let list = this.drawerInfo.calendarDayVOList.filter(v => v.modelId === item)
                        let ext = {
                            modelId: item,
                            // isHighPriority: list[0].isHighPriority,
                        }
                        this.setDayRange(list, ext)
                    })
                }, 1000)

            }
        },
        changeDate() {
            if (this.checkboxGroup1.length > 0) {
                this.setDay()
            } else {

            }
        },
        change(val) {
            this.checkboxGroup1 = []
            if (val?.some(v => v === 0)) {
                this.checkboxGroup1.push(...[1, 2, 3, 4, 5])
            }
            if (val?.some(v => v === 1)) {
                this.checkboxGroup1.push(...[0, 6])
            }
        },
        setDay() {
            let currentYear = moment(this.form.year, 'YYYY').format('YYYY') - 0
            this.months.forEach((item, i) => {
                let start = moment(`${currentYear}-${i + 1}-01`)
                let end = i == 11 ? moment(`${currentYear + 1}-${i + 1}-01`) : moment(`${currentYear}-${i + 2}-01`)
                let dates = []
                while (start.unix() < end.unix()) {
                    if (this.checkboxGroup1.includes(start.day()) && this.holiday === 0) {
                        this.pushDate(start) && dates.push(start.format('YYYY-MM-DD'))
                    } else if (this.checkboxGroup1.includes(start.day()) && this.holiday === 1) {
                        if (!this.months[i].attributes[0].dates.includes(start.format('YYYY-MM-DD'))) {
                            this.pushDate(start) && dates.push(start.format('YYYY-MM-DD'))
                        }
                    } else if (this.checkboxGroup1.includes(start.day()) && this.holiday === 2) {
                        if (this.months[i].attributes[0].dates.includes(start.format('YYYY-MM-DD'))) {
                            this.pushDate(start) && dates.push(start.format('YYYY-MM-DD'))
                        }
                    }
                    start.add(1, 'd')
                }
                this.months[i].attributes[1].dates = dates
            })
            console.log(this.months, 'rll')
        },
        setDayRange(list, ext) {
            let currentYear = moment(this.form.year, 'YYYY').format('YYYY') - 0
            this.months.forEach((item, i) => {
                let start = moment(`${currentYear}-${i + 1}-01`)
                let end = i == 11 ? moment(`${currentYear + 1}-${i + 1}-01`) : moment(`${currentYear}-${i + 2}-01`)
                let dates = []
                list.forEach(v => {
                    if (moment(v.calendarDate).unix() >= start.unix() && moment(v.calendarDate).unix() < end.unix()) {
                        dates.push(v)
                    }
                })
                if (dates.length > 0) {
                    let ban = this.shiftList.find(v => v.id === ext.modelId)
                    this.months[i].attributes.push({
                        dates: dates.map(v => v.calendarDate),
                        ...ext,
                        // highlight: color.stringToColor(ext.modelId),
                        highlight: {
                            style: {
                                'background-color': ban.colour
                            }
                        },
                        popover: {
                            label: `${ban?.code}-${ban?.name}`
                        },
                    })
                    // 优先
                    console.log('------------->>>>', dates)
                    let hiIndex = this.months[i].attributes.findIndex(v => v.bar)
                    if (hiIndex > -1) {
                        this.months[i].attributes[hiIndex].dates.push(...dates.filter(v => v.isHighPriority === 1).map(v => v.calendarDate))
                    } else {
                        this.months[i].attributes.push({
                            dates: dates.filter(v => v.isHighPriority === 1).map(v => v.calendarDate),
                            bar: {
                                style: {
                                    backgroundColor: 'red',
                                }
                            },
                        })
                    }

                    this.months[i].attributes[1] = {
                        highlight: true,
                        dates: []
                    }
                }
            })
            console.log(this.months, 'rllllllll')
        },
        pushDate(start) {
            if (start.unix() >= moment(this.form.date[0]).unix() && start.unix() <= moment(this.form.date[1]).unix()) {
                return true
            }
            return false
        },
        changeHigh(val) {
            console.log('------------changeHigh')
            if (this.currentDateItem) {
                // 设置优先
                let index = this.currentDateItem.attributes.findIndex(item => !!item.bar)
                if (val) {
                    if (index === -1) {
                        this.currentDateItem.attributes.push({
                            bar: {
                                style: {
                                    backgroundColor: 'red',
                                }
                            },
                            dates: [this.currentDate]
                        })
                    } else {
                        this.currentDateItem.attributes[index].dates.push(this.currentDate)
                    }
                } else {
                    let i = this.currentDateItem.attributes[index].dates.findIndex(v => v === this.currentDate)
                    this.currentDateItem.attributes[index].dates.splice(i, 1)
                }

            } else {
                this.isHighPriority = false
            }
            console.log(this.months)
        },
        dayclick(day, item) {
            let index = item.attributes[1].dates.findIndex(v => v === day.id)
            if (index >= 0) {
                item.attributes[1].dates.splice(index, 1)
            } else {
                item.attributes[1].dates.push(day.id)
            }
            let list = this.getPlanDate()
            let highDates = this.getHighDate()
            this.currentDate = day.id
            if (list.find(v => v.calendarDate === day.id)) {
                this.currentDateItem = item
                if (highDates.indexOf(this.currentDate) > -1) {
                    this.isHighPriority = true
                } else {
                    this.isHighPriority = false
                }
            } else {
                this.currentDateItem = null
                this.isHighPriority = false
            }

            console.log('------currentDate', this.currentDate, item, highDates)
        },
        getHighDate() {
            let list = []
            this.months.forEach(item => {
                item.attributes.forEach((v, i) => {
                    if (i > 1 && v.bar) {
                        v.dates.forEach(child => {
                            list.push(child)
                        })

                    }
                })
            })
            return list
        },
        getPlanDate() {
            // 优先级
            let list = []
            this.months.forEach(item => {
                item.attributes.forEach((v, i) => {
                    if (i > 1 && v.modelId) {

                        v.dates.forEach(child => {
                            // 重复排班去重
                            let repeatIndex = list.findIndex(v => v.calendarDate === child)
                            if (repeatIndex > -1) {
                                list.splice(repeatIndex, 1, {
                                    modelId: v.modelId,
                                    // isHighPriority: v.isHighPriority,
                                    calendarDate: child,
                                    ...this.setOffDay(child)
                                })
                            } else {
                                list.push({
                                    modelId: v.modelId,
                                    // isHighPriority: v.isHighPriority,
                                    calendarDate: child,
                                    ...this.setOffDay(child)
                                })
                            }

                        })

                    }
                })
            })
            return list
        },
        plan() {
            if (!this.modelId) {
                return this.$message.warning('请选择班次方案');
            }
            let dateDTOList = this.months.map(item => item.attributes[1].dates)
            console.log('------plan', dateDTOList)
            let list = dateDTOList.flat(2).map(v => {
                return {
                    modelId: this.modelId,
                    // isHighPriority: this.isHighPriority ? 1 : 0,
                    calendarDate: v,
                    ...this.setOffDay(v)
                }
            })
            let ext = {
                modelId: this.modelId,
                // isHighPriority: this.isHighPriority ? 1 : 0,
            }

            this.setDayRange(list, ext)
        },
        setOffDay(time) {
            let { id: offDayId } = this.offDays.find(v => moment(time).unix() >= moment(v.startOffDay).unix() && moment(time).unix() <= moment(v.endOffDay).unix()) || {}
            let idOffDay = offDayId ? 1 : 0
            return {
                idOffDay,
                offDayId
            }
        },
        planCancel() {
            // 取消排班只能取消当前时间之后的排班
            let now = moment(moment().format('YYYY-MM-DD')).valueOf()
            this.months.forEach((item, i) => {
                // item.attributes = item.attributes.slice(0, 2)
                item.attributes = item.attributes.map((v, i) => {
                    if (i > 1) {
                        v.dates = v.dates.filter(c => {
                            if (moment(c).valueOf() <= now) {
                                return true
                            }
                        })
                    }
                    return v
                })
            })
        },
        save() {
            let highDates = this.getHighDate()
            let list = this.getPlanDate().map(v => {
                if (highDates.indexOf(v.calendarDate) > -1) {
                    return {
                        ...v,
                        isHighPriority: 1
                    }
                } else {
                    return {
                        ...v,
                        isHighPriority: 0
                    }
                }
            })
            this.$refs.dialogForm.validate(async (valid) => {
                if (valid) {
                    if (list.length == 0) {
                        return this.$message.warning('未排班');
                    }
                    if (!this.drawerInfo.id) {
                        let data = {
                            codeRuleId: '',
                            code: this.form.code,
                            name: this.form.name,
                            year: this.form.year,
                            status: 1,
                            dateDTOList: list
                        }
                        this.$HTTP.post('/api/smis/calendar', data).then(res => {
                            if (res.code === 200) {
                                this.$emit('success')
                            } else {
                                this.$message.warning(res.msg);
                            }
                        })
                    } else {
                        let data = {
                            id: this.drawerInfo.id,
                            dateDTOList: list
                        }
                        this.$HTTP.put('/api/smis/calendar', data).then(res => {
                            if (res.code === 200) {
                                this.$emit('success')
                            } else {
                                this.$message.warning(res.msg);
                            }
                        })
                    }

                } else {
                    this.$message.warning('请检查必填项');
                }
            })
        }
    }
}
</script>

<style lang="scss" scoped></style>